// *******************************************************
//  Strings used by SOAP
//
// 1.5
// 2002/01/24 21:27:29
// 1.7
// 2002/03/20 17:55:19
//
// NOTE: Strings in the const section should not be
//       localized. Strings in the resourceString section
//       should be localized as necessary.
// *******************************************************
unit WSDLImpConst;

{$IFDEF CONDITIONALEXPRESSIONS}
  {$IF CompilerVersion >= 18.5}
    {$DEFINE SPACELY_UP}
  {$IFEND}
  {$IF CompilerVersion >= 19.0}
    {$DEFINE HIGHLANDER_UP}
  {$IFEND}
{$ENDIF}

interface

uses WSDLModelIntf;

{$IFDEF LINUX}
{ Linux linebreaks don't work in code previewer }
const

  SLineBreak = #13#10;

{$ENDIF}

{$DEFINE USE_FREE_AND_NIL}


type

  TIntfImplOption = (iiCBuilder, iiPascal, iiPascalIntf, iiPascalImpl, iiCBuilderIntf, iiCBuilderImpl);

const

sCommentDivider =
'//---------------------------------------------------------------------------' + sLineBreak;

{ C++ specific constants }

// Include statement of an Imported SOAP header
//
sInclude_WS = '#include "%s.h"'            + sLineBreak;

// Standard Include used in .CPP files
//
sStandardSourceInclude =
                     '#pragma hdrstop'                         + sLineBreak +
                                                                 sLineBreak +
                     '#if !defined(%0:sH)'                     + sLineBreak +
                     '#include "%0:s.h"'                       + sLineBreak +
                     '#endif'                                  + sLineBreak + sLineBreak;
sStandardSourceIncludeVCL  =
                     '#include <vcl.h>'                        + sLineBreak + sStandardSourceInclude;
sStandardSourceIncludeCLX  =
                     '#include <clx.h>'                        + sLineBreak + sStandardSourceInclude;

// Guard used in Headers
//
sHdrGuardBegin     = '#ifndef   %0:sH'                         + sLineBreak +
                     '#define   %0:sH'                         + sLineBreak +
                                                                 sLineBreak +
                     '#include <System.hpp>'                   + sLineBreak +
                     '#include <InvokeRegistry.hpp>'           + sLineBreak +
                     '#include <XSBuiltIns.hpp>'               + sLineBreak;

sClientHeaders     = '#include <SOAPHTTPClient.hpp>'           + sLineBreak + sLineBreak;

sCppSOAPMacros     = '#if !defined(SOAP_REMOTABLE_CLASS)'      + sLineBreak +
                     '#define SOAP_REMOTABLE_CLASS __declspec(delphiclass)' + sLineBreak +
                     '#endif'                                  + sLineBreak +
                     '#if !defined(AS_UNBOUNDED)'              + sLineBreak +
                     '#define AS_UNBOUNDED false'              + sLineBreak +
                     '#endif'                                  + sLineBreak +
                     '#if !defined(AS_ATTRIBUTE)'              + sLineBreak +
                     '#define AS_ATTRIBUTE false'              + sLineBreak +
                     '#endif'                                  + sLineBreak;

sHdrGuardEnd       =                                             sLineBreak +
                     '#endif // %0:sH'                         + sLineBreak;

// Namespace block in Headers
//
sNamespaceHdrBegin = sLineBreak +
                     'namespace NS_%0:s {'                     + sLineBreak + sLineBreak;
sNamespaceHdrEnd   = sLineBreak +
                     '};     // NS_%0:s'                       + sLineBreak + sLineBreak;

sNamespaceUsing    = '#if !defined(NO_IMPLICIT_NAMESPACE_USE)' + sLineBreak +
                     'using  namespace NS_%0:s;'               + sLineBreak +
                     '#endif'                                  + sLineBreak + sLineBreak;

// Namespace block in Source
//
sNamespaceSrcBegin = 'namespace NS_%s {'                       + sLineBreak;
sNamespaceSrcEnd   = sLineBreak +
                     '};     // NS_%s'                         + sLineBreak;

sInterfaceDecl     = '__interface INTERFACE_UUID("%s") %s : public %s'  + sLineBreak +
                     '{'                                       + sLineBreak +
                     'public:'                                 + sLineBreak;
sInterfaceDeclEnd  = '};'                                      + sLineBreak +
                     'typedef DelphiInterface<%0:s> _di_%0:s;' + sLineBreak
                                                               + sLineBreak;
sIntfFactoryDecl   = '_di_%0:s Get%0:s(bool useWSDL=false, AnsiString addr="", THTTPRIO* HTTPRIO=0);' + sLineBreak + sLineBreak;
sImplDecl          = 'class T%0:sImpl : public TInvokableClass, public %0:s'+ sLineBreak +
                     '{'                                       + sLineBreak +
                     'public:'                                 + sLineBreak + sLineBreak +
                     '  /* %0:s */'                            + sLineBreak;
sImplOther         = sLineBreak +
                     '  /* IUnknown */'                                                                    + sLineBreak +
                     '  HRESULT STDMETHODCALLTYPE QueryInterface(const GUID& IID, void **Obj)'             + sLineBreak +
                     '                        { return GetInterface(IID, Obj) ? S_OK : E_NOINTERFACE; }'   + sLineBreak +
                     '  ULONG STDMETHODCALLTYPE AddRef() { return TInterfacedObject::_AddRef();  }'        + sLineBreak +
                     '  ULONG STDMETHODCALLTYPE Release(){ return TInterfacedObject::_Release(); }'        + sLineBreak;
sImplEnd           = '};'                                      + sLineBreak + sLineBreak;
sMethImpl          = '{'                                                        + sLineBreak +
                     '  /* TODO : Implement WebService method %s */'            + sLineBreak +
                     '%s}'                                                      + sLineBreak + sLineBreak;
sReturnDecl        = '  %s %s;'                                                 + sLineBreak;
sReturnTempl       = '  return %s;'                                             + sLineBreak;
sReturnNewTempl    = '  return new %s();'                                       + sLineBreak;

sFactoryImpl       = '_di_%0:s Get%0:s(bool useWSDL, AnsiString addr, THTTPRIO* HTTPRIO)' + sLineBreak +
                     '{'                                                        + sLineBreak +
                     '  static const char* defWSDL= "%1:s";'                    + sLineBreak +
                     '  static const char* defURL = "%2:s";'                    + sLineBreak +
                     '  static const char* defSvc = "%3:s";'                    + sLineBreak +
                     '  static const char* defPrt = "%4:s";'                    + sLineBreak +
                     '  if (addr=="")'                                          + sLineBreak +
                     '    addr = useWSDL ? defWSDL : defURL;'                   + sLineBreak +
                     '  THTTPRIO* rio = HTTPRIO ? HTTPRIO : new THTTPRIO(0);'   + sLineBreak +
                     '  if (useWSDL) {'                                         + sLineBreak +
                     '    rio->WSDLLocation = addr;'                            + sLineBreak +
                     '    rio->Service = defSvc;'                               + sLineBreak +
                     '    rio->Port = defPrt;'                                  + sLineBreak +
                     '  } else {'                                               + sLineBreak +
                     '    rio->URL = addr;'                                     + sLineBreak +
                     '  }'                                                      + sLineBreak +
                     '  _di_%0:s service;'                                      + sLineBreak +
                     '  rio->QueryInterface(service);'                          + sLineBreak +
                     '  if (!service && !HTTPRIO)'                                          + sLineBreak +
                     '    delete rio;'                                          + sLineBreak +
                     '  return service;'                                        + sLineBreak +
                     '}'                                                        + sLineBreak + sLineBreak;

sInitRoutineBeg    = 'static void RegTypes()'                  + sLineBreak +
                     '{'                                       + sLineBreak;

sInitRoutineEnd    = '}'                                       + sLineBreak +
                     '#pragma startup RegTypes 32'             + sLineBreak;

sRegInterface1     = '  InvRegistry()->RegisterInterface(__interfaceTypeinfo(%s), L"%s", L"%s");'   + sLineBreak;
sRegInterface2     = '  InvRegistry()->RegisterInterface(__interfaceTypeinfo(%s), L"%s", L"%s",'    + sLineBreak +
                     '                                   "", L"%s");'                               + sLineBreak;
sRegSOAPAction     = '  InvRegistry()->RegisterDefaultSOAPAction(__interfaceTypeinfo(%s), L"%s");'  + sLineBreak;
sRegAllSOAPAction  = '  InvRegistry()->RegisterAllSOAPActions(__interfaceTypeinfo(%s), L"%s");'     + sLineBreak;

sRegParamNames     = '  InvRegistry()->RegisterReturnParamNames(__interfaceTypeinfo(%s), "%s");'    + sLineBreak;
sRegInvokeOptDoc   = '  InvRegistry()->RegisterInvokeOptions(__interfaceTypeinfo(%s), ioDocument);' + sLineBreak;
sRegInvokeOptLit   = '  InvRegistry()->RegisterInvokeOptions(__interfaceTypeinfo(%s), ioLiteral);'  + sLineBreak;

sRegMethod         = '  InvRegistry()->RegisterExternalMethName(__interfaceTypeinfo(%s), "%s", L"%s");'             + sLineBreak;
sRegParam          = '  InvRegistry()->RegisterExternalParamName(__interfaceTypeinfo(%s), "%s", "%s", L"%s");'      + sLineBreak;

sRegHolderTypeInfo1= '  RemClassRegistry()->RegisterXSInfo(GetClsMemberTypeInfo(__typeinfo(%0:s_TypeInfoHolder)), L"%1:s", L"%0:s");'          + sLineBreak;
sRegHolderTypeInfo2= '  RemClassRegistry()->RegisterXSInfo(GetClsMemberTypeInfo(__typeinfo(%0:s_TypeInfoHolder)), L"%1:s", L"%0:s", L"%2:s");' + sLineBreak;

sRegArrayTypeInfo1 = '  RemClassRegistry()->RegisterXSInfo(__arrayTypeinfo(%s), L"%s", L"%s");'         + sLineBreak;
sRegArrayTypeInfo2 = '  RemClassRegistry()->RegisterXSInfo(__arrayTypeinfo(%s), L"%s", L"%s", L"%s");'  + sLineBreak;

sRegClass1         = '  RemClassRegistry()->RegisterXSClass(__classid(%s), L"%s", L"%s");'              + sLineBreak;
sRegClass2         = '  RemClassRegistry()->RegisterXSClass(__classid(%s), L"%s", L"%s", L"%s");'       + sLineBreak;

sRegImpl           = '  InvRegistry()->RegisterInvokableClass(__classid(T%sImpl));'                     + sLineBreak;

sRegHeaderClass    = '  InvRegistry()->RegisterHeaderClass(__classid(T%sImpl), L"%s");'                     + sLineBreak;

sProxyClassDecl    = 'class %0:s_Proxy : public THTTPRIO, public virtual %0:s {'+ sLineBreak;

sTypeInfoHolder    = 'class %0:s_TypeInfoHolder : public TObject {'+ sLineBreak +
                     '  %0:s __instanceType;'     + sLineBreak +
                     'public:'                    + sLineBreak +
                     '__published:'               + sLineBreak +
                     '  __property %0:s __propType = { read=__instanceType };' + sLineBreak +
                     '};'                         + sLineBreak;


sStoredAsAttribute = ', stored = AS_ATTRIBUTE';
sStoredAsUnbounded = ', stored = AS_UNBOUNDED';
                     
sRemoteBaseClass   = 'TRemotable';
sRemoteClassDecl   = 'class %0:s : public %1:s {' + sLineBreak +
                     'private:'                   + sLineBreak;
sRemoteClassPublic = 'public:'                    + sLineBreak;
sRemoteClassPublish= '__published:'               + sLineBreak;
sRemoteClassCtr    = '  __fastcall  %s();'        + sLineBreak;
sRemoteClassDtr    = '  __fastcall ~%s();'        + sLineBreak;

sRemoteClsCtrImpl  = '__fastcall %0:s::%0:s() {'  + sLineBreak +
                     '{'                          + sLineBreak +
                     '%1:s'                       +
                     '}'                          + sLineBreak;

sRemoteClsDtrImpl  = '__fastcall %0:s::~%0:s()'   + sLineBreak +
                     '{'                          + sLineBreak +
                     '%1:s'                       +
                     '}'                          + sLineBreak;

sCtrInitLine       = '%s'+
                     '  %s= new %s();'            + sLineBreak;
sDtrClsLine        = '%s'+
                     '  delete %s;'               + sLineBreak;
sDtrArrayLine      = '%s'+
                     '  for(int i=0; i<%1:s.Length; i++)'      + sLineBreak +
                     '    if (%1:s[i])'                        + sLineBreak +
                     '      delete %1:s[i];'      + sLineBreak;
sCtrDtrEnd         = '}'                          + sLineBreak;
sRemoteClassDeclEnd= '};'                         + sLineBreak;

sOperationInfo     = '  // %s';

sCppComment        = '/* %s */';
sPasComment        = '{ %s }';
sLineComment       = '// %s';

sCppCommentIdent   = '  /* %s */';
sPasCommentIdent   = '  { %s }';

sPasCommentLF      = '{ %s }'  + sLineBreak;

sTypeError         = '[TYPE ERROR: %s]';

sPortTypeSignature = 'Name(%s), Namespace(%s), BindingName(%s), Service(%s), Port(%s)';

{ Pascal constants }
sUDDIOperator   = 'sUDDIOperator';
sUDDIBindingKey = 'sUDDIBindingKey';

sUnitBeg  =  'unit %0:s;'                                       + sLineBreak +
                                                                  sLineBreak +
             'interface'                                        + sLineBreak +
                                                                  sLineBreak +
             'uses InvokeRegistry, SOAPHTTPClient, Types, XSBuiltIns%s;'     + sLineBreak;
sConstBeg  =                                                      sLineBreak +
             'const'                                            + sLineBreak;

{ NOTE: This is temporary. Ideally it would be defined in the SOAP runtime,
        same as 'AS_ATTRIBUTE' but that would require an interface change }
sINLINE_CONSTS= '  IS_OPTN = $0001;'                            + sLineBreak +
                '  IS_UNBD = $0002;'                            + sLineBreak +
                '  IS_NLBL = $0004;'                            + sLineBreak +
                '  IS_UNQL = $0008;'                            + sLineBreak +
                '  IS_ATTR = $0010;'                            + sLineBreak +
                '  IS_TEXT = $0020;'                            + sLineBreak;

sIS_OPTN      = 'IS_OPTN';
sIS_UNBD      = 'IS_UNBD';
sIS_NLBL      = 'IS_NLBL';
sIS_UNQL      = 'IS_UNQL';
sIS_ATTR      = 'IS_ATTR';
sIS_TEXT      = 'IS_TEXT';

sUDDIConst    = '  '+sUDDIOperator+'   = ''%s'';'               + sLineBreak +
                '  '+sUDDIBindingKey+' = ''%s'';'               + sLineBreak;

sTypeBeg  =                                                       sLineBreak +
             'type'                                             + sLineBreak + sLineBreak;

sSOAPMidas = ', SOAPMidas';

sUnitImpl =  sLineBreak + sLineBreak +
             'implementation'                                   + sLineBreak;
sUnitUses =  '  uses SysUtils;'                                 + sLineBreak +
                                                                  sLineBreak;
sUnitType =  'type'                                             + sLineBreak +
                                                                  sLineBreak;

sRegProcPrefix = 'RegisterTypeProc';
sRegProcStart  = 'procedure '+sRegProcPrefix+'%d;'              + sLineBreak +
                 'begin'                                        + sLineBreak;
sRegProcEnd    = 'end;'                                         + sLineBreak
                                                                + sLineBreak;
sRegProcCall   =  '  '+sRegProcPrefix+'%d;'                     + sLineBreak;
sUnitInit =  'initialization'                                   + sLineBreak;

sUnitEnd  =  sLineBreak +
             'end.';

sRemoteClassDeclPas   = '  %0:s = class(%1:s)'         + sLineBreak +
                        '  private'                    + sLineBreak;
sRemoteClassPublicPas = '  public'                     + sLineBreak;
sRemoteClassPublishPas= '  published'                  + sLineBreak;
sRemoteClassCtrPas    = '    constructor Create; override;'      + sLineBreak;
sRemoteClassDtrPas    = '    destructor Destroy; override;'      + sLineBreak;
sRemoteClassSetterPas = '    procedure Set%0:s(const A%1:s: %1:s);'    + sLineBreak;
sRemoteClassGetterPas = '    function  Get%0:s: %1:s;'                 + sLineBreak;
sRemoteClassSetterIdx = '    procedure Set%0:s(Index: Integer; const A%1:s: %1:s);'    + sLineBreak;
sRemoteClassGetterIdx = '    function  Get%0:s(Index: Integer): %1:s;'                 + sLineBreak;
sSpecifiedField       = '    F%s_Specified: boolean;';
sSpecifiedStoredProcF = ' stored F%s_Specified';
sSpecifiedStoredProcI = ' stored %s_Specified';
//sSpecifiedStoredProcDecl =
//                        '    function  %s_Value_Specified: boolean;'   + sLineBreak;

sSpecifiedStoredProcDeclIdx =
                        '    function  %s_Specified(Index: Integer): boolean;'   + sLineBreak;

//sSpecifiedStoredProcImpl  = 'function %0:s.%1:s_Specified: boolean;'        + sLineBreak +
//                            'begin'                                         + sLineBreak +
//                            '  Result := F%1:s_Specified;'                  + sLineBreak +
//                            'end;'                                          + sLineBreak;

sSpecifiedStoredProcImplIdx='function %0:s.%1:s_Specified(Index: Integer): boolean;'  + sLineBreak +
                            'begin'                                         + sLineBreak +
                            '  Result := F%1:s_Specified;'                  + sLineBreak +
                            'end;'                                          + sLineBreak;
                            
sRemoteClassSetterImplPas = 'procedure %0:s.Set%1:s(const A%2:s: %2:s);'    + sLineBreak +
                            'begin'                                         + sLineBreak +
                            '  F%1:s := A%2:s;'                             + sLineBreak +
                            'end;'                                          + sLineBreak;

sRemoteClassSetterImplPas2= 'procedure %0:s.Set%1:s(const A%2:s: %2:s);'    + sLineBreak +
                            'begin'                                         + sLineBreak +
                            '  F%1:s := A%2:s;'                             + sLineBreak +
                            '  F%1:s_Specified := True;'                    + sLineBreak +
                            'end;'                                          + sLineBreak;

sRemoteClassGetterImplPas = 'function %0:s.Get%1:s: %2:s;'                  + sLineBreak +
                            'begin'                                         + sLineBreak +
                            '  Result := F%1:s;'                            + sLineBreak +
                            'end;'                                          + sLineBreak;

sRemoteClassSetterImplIdx = 'procedure %0:s.Set%1:s(Index: Integer; const A%2:s: %2:s);'    + sLineBreak +
                            'begin'                                         + sLineBreak +
                            '  F%1:s := A%2:s;'                             + sLineBreak +
                            'end;'                                          + sLineBreak;

sRemoteClassSetterImplIdx2= 'procedure %0:s.Set%1:s(Index: Integer; const A%2:s: %2:s);'    + sLineBreak +
                            'begin'                                         + sLineBreak +
                            '  F%1:s := A%2:s;'                             + sLineBreak +
                            '  F%1:s_Specified := True;'                    + sLineBreak +
                            'end;'                                          + sLineBreak;

sRemoteClassGetterImplIdx = 'function %0:s.Get%1:s(Index: Integer): %2:s;'                  + sLineBreak +
                            'begin'                                         + sLineBreak +
                            '  Result := F%1:s;'                            + sLineBreak +
                            'end;'                                          + sLineBreak;


sSTORED_AS_ATTRIBUTE  = ' stored False';
sSTORED_AS_UNBOUNDED  = ' stored False';

sArrayAsClassDecl     = '';

sRemoteClassDeclEndPas= '  end;'                       + sLineBreak;
sRemoteClsCtrImplPas1 = 'constructor %0:s.Create;'     + sLineBreak +
                        'begin'                        + sLineBreak +
                        '  inherited Create;'          + sLineBreak;
sRemoteClsCtrImplPas2 = '  FSerializationOptions := [%s];' + sLineBreak;
sRemoteClsCtrImplPas3 = 'end;'                         + sLineBreak;
sRemoteClsDtrImplPas1 = 'destructor %0:s.Destroy;'     + sLineBreak +
                        'begin'                        + sLineBreak;
sRemoteClsDtrImplPas2 = 'destructor %0:s.Destroy;'     + sLineBreak +
                        'var'                          + sLineBreak +
                        '  I: Integer;'                + sLineBreak +
                        'begin'                        + sLineBreak;
{$IFDEF USE_FREE_AND_NIL}
sRemoteClsDtrImplPas3 = '  for I := 0 to Length(%0:s)-1 do'   + sLineBreak +
                        '    FreeAndNil(%0:s[I]);'            + sLineBreak +
                        '  SetLength(%0:s, 0);'               + sLineBreak;
{$ELSE}
sRemoteClsDtrImplPas3 = '  for I := 0 to Length(%0:s)-1 do'   + sLineBreak +
                        '    if Assigned(%0:s[I]) then'       + sLineBreak +
                        '      %0:s[I].Free;'                 + sLineBreak +
                        '  SetLength(%0:s, 0);'               + sLineBreak;
{$ENDIF}
{$IFDEF USE_FREE_AND_NIL}
sRemoteClsDtrImplPas4 = '  FreeAndNil(%0:s);'          + sLineBreak;
{$ELSE}
sRemoteClsDtrImplPas4 = '  if Assigned(%0:s) then'     + sLineBreak +
                        '    %0:s.Free;'               + sLineBreak;
{$ENDIF}
sRemoteClsDtrImplPas5 = '  inherited Destroy;'         + sLineBreak +
                        'end;'                         + sLineBreak;

sInterfaceDeclPas     = '  %s = interface(%s)'                      + sLineBreak +
                        '  [''%s'']'                                + sLineBreak;
sInterfaceDeclEndPas  = '  end;'                                    + sLineBreak + sLineBreak;

{ For cases where WSDL has Binding }
sIntfFactoryDeclPas1  = 'function Get%0:s(UseWSDL: Boolean=System.False; Addr: string=''''; HTTPRIO: THTTPRIO = nil): %0:s;'  + sLineBreak;
sFactoryImplPas1      = 'function Get%0:s(UseWSDL: Boolean; Addr: string; HTTPRIO: THTTPRIO): %0:s;'  + sLineBreak +
                        'const'                                                    + sLineBreak +
                        '  defWSDL = ''%1:s'';'                                    + sLineBreak +
                        '  defURL  = ''%2:s'';'                                    + sLineBreak +
                        '  defSvc  = ''%3:s'';'                                    + sLineBreak +
                        '  defPrt  = ''%4:s'';'                                    + sLineBreak +
                        'var'                                                      + sLineBreak +
                        '  RIO: THTTPRIO;'                                         + sLineBreak +
                        'begin'                                                    + sLineBreak +
                        '  Result := nil;'                                         + sLineBreak +
                        '  if (Addr = '''') then'                                  + sLineBreak +
                        '  begin'                                                  + sLineBreak +
                        '    if UseWSDL then'                                      + sLineBreak +
                        '      Addr := defWSDL'                                    + sLineBreak +
                        '    else'                                                 + sLineBreak +
                        '      Addr := defURL;'                                    + sLineBreak +
                        '  end;'                                                   + sLineBreak +
                        '  if HTTPRIO = nil then'                                  + sLineBreak +
                        '    RIO := THTTPRIO.Create(nil)'                          + sLineBreak +
                        '  else'                                                   + sLineBreak +
                        '    RIO := HTTPRIO;'                                      + sLineBreak +
                        '  try'                                                    + sLineBreak +
                        '    Result := (RIO as %0:s);'                             + sLineBreak +
                        '    if UseWSDL then'                                      + sLineBreak +
                        '    begin'                                                + sLineBreak +
                        '      RIO.WSDLLocation := Addr;'                          + sLineBreak +
                        '      RIO.Service := defSvc;'                             + sLineBreak +
                        '      RIO.Port := defPrt;'                                + sLineBreak +
                        '    end else'                                             + sLineBreak +
                        '      RIO.URL := Addr;'                                   + sLineBreak +
                        '  finally'                                                + sLineBreak +
                        '    if (Result = nil) and (HTTPRIO = nil) then'           + sLineBreak +
                        '      RIO.Free;'                                          + sLineBreak +
                        '  end;'                                                   + sLineBreak +
                        'end;'                                                     + sLineBreak +
                                                                                    sLineBreak;
{ For cases where we have a URL but no binding }
sIntfFactoryDeclPas2  = 'function Get%0:s(const Addr: string=''''): %0:s;'         + sLineBreak;
sFactoryImplPas2      = 'function Get%0:s(const Addr: string): %0:s;'              + sLineBreak +
                        'const'                                                    + sLineBreak +
                        '  defURL  = ''%1:s'';'                                    + sLineBreak +
                        'var'                                                      + sLineBreak +
                        '  RIO: THTTPRIO;'                                         + sLineBreak +
                        'begin'                                                    + sLineBreak +
                        '  Result := nil;'                                         + sLineBreak +
                        '  RIO := THTTPRIO.Create(nil);'                           + sLineBreak +
                        '  try'                                                    + sLineBreak +
                        '    Result := (RIO as %0:s);'                             + sLineBreak +
                        '    if (Addr = '''') then'                                + sLineBreak +
                        '      RIO.URL := defURL'                                  + sLineBreak +
                        '    else'                                                 + sLineBreak +
                        '      RIO.URL := Addr;'                                   + sLineBreak +
                        '  finally'                                                + sLineBreak +
                        '    if Result = nil then'                                 + sLineBreak +
                        '      RIO.Free;'                                          + sLineBreak +
                        '  end;'                                                   + sLineBreak +
                        'end;'                                                     + sLineBreak +
                                                                                    sLineBreak;
sImplDeclPas          = '  %0:sImpl = class(TInvokableClass, %0:s)' + sLineBreak +
                        '  public'                                  + sLineBreak +
                        '    { %0:s }'                              + sLineBreak;
sImplDeclPas2         = '  T%0:s = class(TInvokableClass, %0:s)'     + sLineBreak +
                        '  public'                                  + sLineBreak +
                        '    { %0:s }'                              + sLineBreak;
sImplDeclEndPas       = '  end;'                                    + sLineBreak + sLineBreak;

sImplSuffix           = 'Impl';

sIntfSuffix           = 'Intf';

sDummyImplPas         = 'begin'                                     + sLineBreak +
                        '  { TODO - Implement method %s }'          + sLineBreak +
                        'end;'                                      + sLineBreak + sLineBreak;

{0: Element Type Name }
sDefaultArrayDeclPas  = '    function   Get%0:sArray(Index: Integer): %0:s;       '           + sLineBreak +
                        '    procedure  Set%0:sArray(Index: Integer; const Item: %0:s);'      + sLineBreak +
                        '    function   Get%0:sArrayLength: Integer;'                         + sLineBreak +
                        '    procedure  Set%0:sArrayLength(Len: Integer);'                    + sLineBreak +
                                                                                                sLineBreak +
                        '    property   %0:sArray[Index: Integer]: %0:s read Get%0:sArray write Set%0:sArray; default;' + sLineBreak +
                        '    property   Len: Integer read Get%0:sArrayLength write Set%0:sArrayLength;'                 + sLineBreak;

{ 0: Class Type Name, 1: Element Type Name, 2: Member Name }
sDefaultArrayImplPas  = 'function %0:s.Get%1:sArray(Index: Integer): %1:s;' + sLineBreak +
                        'begin'                                                           + sLineBreak +
                        '  Result := F%2:s[Index];'                                       + sLineBreak +
                        'end;'                                                            + sLineBreak +
                                                                                            sLineBreak +
                        'procedure %0:s.Set%1:sArray(Index: Integer; const Item: %1:s);'  + sLineBreak +
                        'begin'                                                           + sLineBreak +
                        '   F%2:s[Index] := Item;'                                        + sLineBreak +
                        'end;'                                                            + sLineBreak +
                                                                                            sLineBreak +
                        'function %0:s.Get%1:sArrayLength: Integer;'                      + sLineBreak +
                        'begin'                                                           + sLineBreak +
                        '  if Assigned(F%2:s) then'                                       + sLineBreak +
                        '    Result := Length(F%2:s)'                                     + sLineBreak +
                        '  else'                                                          + sLineBreak +
                        '    Result := 0;'                                                + sLineBreak +
                        'end;'                                                            + sLineBreak +
                                                                                            sLineBreak +
                        'procedure %0:s.Set%1:sArrayLength(Len: Integer);'                + sLineBreak +
                        'begin'                                                           + sLineBreak +
                        '  SetLength(F%2:s, Len);'                                        + sLineBreak +
                        'end;'                                                            + sLineBreak + sLineBreak;

sRegInterfacePas3     = '  InvRegistry.RegisterInterface(TypeInfo(%s), ''%s'', ''%s'', '''', ''%s'', ''%s'');'  + sLineBreak;
sRegInterfacePas2     = '  InvRegistry.RegisterInterface(TypeInfo(%s), ''%s'', ''%s'', '''', ''%s'');'          + sLineBreak;
sRegInterfacePas1     = '  InvRegistry.RegisterInterface(TypeInfo(%s), ''%s'', ''%s'');'                        + sLineBreak;
sRegInvokableClass    = '  InvRegistry.RegisterInvokableClass(%s);'                                             + sLineBreak;

sRegSOAPActionPas     = '  InvRegistry.RegisterDefaultSOAPAction(TypeInfo(%s), ''%s'');'                        + sLineBreak;
sRegAllSOAPActionPas  = '  InvRegistry.RegisterAllSOAPActions(TypeInfo(%s), %s);'                               + sLineBreak;

sRegUDDIInfoPas       = '  InvRegistry.RegisterUDDIInfo(TypeInfo(%s), ''%s'', ''%s'');'                         + sLineBreak;
sRegParamNamesPasBeg  = '  InvRegistry.RegisterReturnParamNames(TypeInfo(%s), ';

sRegInvokeOptDocPas   = '  InvRegistry.RegisterInvokeOptions(TypeInfo(%s), ioDocument);'                        + sLineBreak;
sRegInvokeOptLitPas   = '  InvRegistry.RegisterInvokeOptions(TypeInfo(%s), ioLiteral);'                         + sLineBreak;
sRegInvokeOptUDDIPas  = '  InvRegistry.RegisterInvokeOptions(TypeInfo(%s), ioHasUDDIInfo);'                     + sLineBreak;

sRegMethodPas         = '  InvRegistry.RegisterExternalMethName(TypeInfo(%s), ''%s'', ''%s'');'             + sLineBreak;
sRegParamPas          = '  InvRegistry.RegisterExternalParamName(TypeInfo(%s), ''%s'', ''%s'', ''%s'');'    + sLineBreak;

{ Class, Namespace, Name, ExternalName }
sRegClassPas2         = '  RemClassRegistry.RegisterXSClass(%s, ''%s'', ''%s'', ''%s'');'           + sLineBreak;
{ Class, Namespace, Name }
sRegClassPas1         = '  RemClassRegistry.RegisterXSClass(%s, ''%s'', ''%s'');'                   + sLineBreak;
{ Interface, Namespace, Name, ExternalName }
sRegTypePas2          = '  RemClassRegistry.RegisterXSInfo(TypeInfo(%s), ''%s'', ''%s'', ''%s'');'  + sLineBreak;
{ Interface, Namespace, Name }
sRegTypePas1          = '  RemClassRegistry.RegisterXSInfo(TypeInfo(%s), ''%s'', ''%s'');'          + sLineBreak;

sRegMemberRenamedPas  = '  RemClassRegistry.RegisterExternalPropName(TypeInfo(%s), ''%s'', ''%s'');'+ sLineBreak;

sRegClassSerialOptsPas= '  RemClassRegistry.RegisterSerializeOptions(%s, %s);'                      + sLineBreak;
sRegInfoSerialOptsPas = '  RemClassRegistry.RegisterSerializeOptions(TypeInfo(%s), %s);'            + sLineBreak;

sRegHeaderPas1        = '  InvRegistry.RegisterHeaderClass(TypeInfo(%s), %s, %s, %s);'              + sLineBreak;
sRegHeaderPas2        = '  InvRegistry.RegisterHeaderClass(TypeInfo(%s), %s, ''%s'', ''%s'');'      + sLineBreak;
sRegHeaderMethPas     = '  InvRegistry.RegisterHeaderMethod(TypeInfo(%s), %s, ''%s'', %s, %s);'     + sLineBreak;

sRegFaultPas          = '  InvRegistry.RegisterException(TypeInfo(%s), %s);'                        + sLineBreak;
sRegFaultMethPas      = '  InvRegistry.RegisterExceptionMethod(TypeInfo(%s), %s, ''%s'');'          + sLineBreak;

sUnitSuffix           = '_WS';

SPasFileExt           = '.pas';
SCppFileExt           = '.cpp';
SHFileExt             = '.h';
{$IFDEF MSWINDOWS}
SDfmExt               = '.dfm';
{$ENDIF}
{$IFDEF LINUX}
SDfmExt               = '.xfm';
{$ENDIF}

XMLItemInfoComment: array[XMLItemInfo] of string = ('global',
                                                    'predefined',
                                                    '<element>',
                                                    '<complexType>',
                                                    '<simpleType>',
                                                    '<attribute>',
                                                    'unbounded',
                                                    'alias',
                                                    'nillable'
                                                );

resourcestring

{ Wizard Reg }
SNewXMLDataBinding = '  XML';

{ Wizard Form }
SWSDLDefExt        = '.wsdl';
SSourceFilter      = 'WSDL- (*.wsdl, *.xml)|*.wsdl;*.xml|XML- (*.xml)|*.xml|  (*.*)|*.*';

SConfirmClose      = ' ,    WSDL Importer ?';
sNoBindingsFound   = '    (Bindings)  ';


sWSDLInfoBeg =
   '// ************************************************************************ //' + #13#10 +
   '//        ,  '      + #13#10 +
   '// WSDL-,  :'                                                  + #13#10 +
   '// WSDL      : %0:s'                                                             + #13#10;

sImportInfo =
   '// >   : %s' + #13#10;

sEncodingInfo =
   '//  : %s' + #13#10;

sVersionInfo =
   '//     : %s' + #13#10;

sSchemaRef =
   '// SchemaRef : %s' + #13#10;

sServices =
   '//     : %s' + #13#10;

sPorts =
   '//      : [Srvc %d] - %s' + #13#10;

sGenOptions =
   '// Codegen  : %s' + #13#10;

sWSDLInfoEnd =
   '// (%s - %s)'+ #13#10 +
   '// ************************************************************************ //' + #13#10 + #13#10;

sInfoBeg =
   '// ************************************************************************ //';

sInfoEnd =
   '// ************************************************************************ //' + #13#10;

sPredefinedInfo =
   '%0:s//  ,      WSDL ,  '     + #13#10 +
   '%0:s//   .    [@]  ,     ,'+ #13#10 +
   '%0:s//   [!]   .    '        + #13#10 +
   '%0:s//   / XML  Borland ; ,   '   + #13#10 +
   '%0:s//   WSDL ,       .'   + #13#10;

sPredefinedTypes =
   '// !:%-15s - "%s"';

sDuplicateTypes =
   '// @:%-15s - "%s"';

sWarningInfoBegin =
   '%0:s// []******************************************************** //';

sWarningInfo =
   '%0:s// %1:s';

sWarningInfoEnd =
   '%0:s// ************************************************************************ //';

sRegProcInfo =
   '// ************************************************************************ //' + #13#10 +
   '//      ,   '     + #13#10 +
   '// SOAP.'                                                                    + #13#10;

sBindInfoGeneric =
   '//       : %s';

sExternalName =
   '//        : %s';

sSerialization =
   '// Serializtn: [%s]';

sXMLInfo       =
   '// XML       : %s';

sNamespaceInfo =
   '// Namespace : %s';

sSOAPActionInfo =
   '// soapAction: %s';

sTransportInfo =
   '//  : %s';

sStyleInfo =
   '//      : %s';

sBindingInfo =
   '// binding   : %s';

sServiceInf =
   '//     : %s';

sPortInfo =
   '//       : %s';

sURLInfo =
   '// URL       : %s';   

sServerImpl  =  '%s -   ';

sBindInfoLocalName =
   '// LocalName : %s';

sBindInfoBaseType =
   '// BaseType  : %s';

sBindInfoBound =
   '// Bound     : %s';

sForwardDecl    =                                                                 #13#10 +
   '// *********************************************************************//' + #13#10 +
   '//     WSDL                                   ' + #13#10 +
   '// *********************************************************************//' + #13#10;

sFeedbackRead      = ' : %s';
sFeedbackWrite     = ' : %s';
sFeedbackImp       = ' : %s';
sFeedbackUDDI      = 'UDDI   : %s';
sFeedbackSkip      = '(Skip) : %s';
sFeedbackDebug     = '(Debug): %s';
sWritingType       = '(Write): %s';
sReadingType       = '(Read) : %s';
sSortingType       = '(Sort) : %s:%d - %s';
sSwapingType       = '(Swap) : %s:%d <-> %s:%d, (%s:%s)';
sDottedLine        = '------------------------------------------------';

sMissingDefinition = '**: ''%s'' -   <definition>  namespace "%s"';
sGenericTypeError  = '**: ''%s'' - Invalid Type Definition encountered';
sComplexTypeError  = '**: ''%s'' - Complex Type Definition Error, type="%s"';
sBindableTypeError = '**: ''%s'' - %s';
sSchemaError       = '**: ''%s'' -   ';
sGenericError      = '**: %s';
sSummaryFeedback1  = ':   %d  %d.  %d:';
sSummaryFeedback2  = ':   %d  %d ';
sFailureFeedback   =  #9'%s';
sSchemaWarning     = ': ''%s'' -    XML-';
sUnwindWarningIn   = ':    %s()    ';
sUnwindWarningOut  = ':    %s()    ';


sWarningHeader     = '  { ===============  =============== }';
sWarningAttribute  = '  {  -  - :%s, :%s }';

sCircularTypeLink  = '    : %s';

sUnableToSort      = '    ''%s''   !';
sUnableToSortTypes = '    ''%s''  ''%s''   !';

{ Import Options }
sVerboseOpt        = '      ';
sSkipHTTPBindOpt   = ' Porttypes  HTTP Bindings';
sMapNamedArrays    = 'Map pure collections to wrapper class types';
sSkipUnusedTypes   = '   ';
sImportFaults      = '  ';
sValidateEnum      = '  ';
sImportHeaders     = '  ';
sGenTrueGUID       = ' GUID ,  COM API';
sCollapseAliases   = '   ';

sProcessRefSchemas = '    ';
sStrongClassAliases= '     ';
sAllowOutParameters     = '  ';
sUseSettersAndGetters   = ' Setters  Getters  ';
sProcessNillableOptional= '    ';
sUseXSTypeForSimpleNillable = ' TXSString    ';

sCouldNotUnwrapDocLitElementWrapper = '  ';
sCUInputMessageHasMoreThanOnePart = '     ';
sCUOutputMessageHasMoreThanOnePart= '     ';
sCUInputPartRefersToTypeNotElement= '     ';
sCUOutputPartRefersToTypeNotElement='     ';
sCUInputWrapperElementNotSameAsOperationName = ' element wrapper      ';
sCUInputPartNotAComplexType = '  -   ';
sCUOutputPartNotAComplexType= '  -   ';
sCUMoreThanOneStrictlyOutMembersWereFound = '    strictly out ';
sCUTypeNeedsSpecialSerialization = 'Wrapper type   serialization';
sCUOutputElementIsAPureCollection= '  is a pure collection';

sMethodHeaders = '';

const

CannotUnwrapStr: array[CannotUnwrap] of String = (
                                            sCUInputMessageHasMoreThanOnePart,
                                            sCUOutputMessageHasMoreThanOnePart,
                                            sCUInputPartRefersToTypeNotElement,
                                            sCUOutputPartRefersToTypeNotElement,
                                            sCUInputWrapperElementNotSameAsOperationName,
                                            sCUInputPartNotAComplexType,
                                            sCUOutputPartNotAComplexType,
                                            sCUMoreThanOneStrictlyOutMembersWereFound,
                                            sCUTypeNeedsSpecialSerialization,
                                            sCUOutputElementIsAPureCollection
                                                   );

implementation

end.
